#!/usr/bin/php
<?php
$script_path = dirname(__FILE__);
include_once($script_path.'/lib/HttpClient.class.php');

//$DEFAULT_SRV_URL = 'http://127.0.0.1:2082';
$DEFAULT_TIMEOUT = 50;
$DEFAULT_CONF_FILE = '/etc/esia/rcm.ini';
$DEFAULT_TYPE = 'ps1';
$DEFAULT_METHOD = 'POST';

$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

$conf = parse_ini_file($DEFAULT_CONF_FILE,true);
$o_key = $conf['LISTEN']['key'];
$o_server_url = 'http://'.$conf['LISTEN']['address'].':'.$conf['LISTEN']['port'];

//Récupération et vérifications des paramètres.
$type_list = array('inventory','inventory-os','inventory-software','inventory-hardware','load','mem','storage','uptime','time','interface','iodisk','smartctl','service','scheduled_task');

$params = new Parameters();
$params->addOpt('','h','help',NULL,'print this help');
$params->addOpt('','D','debug',NULL,'Use to enable debug mode');
$params->addOpt(':','u','uri','<url>','Powershell daemon url (http://127.0.0.1:2082 by default)');
$params->addOpt(':','H','host','<host>','Host IP ou FQDN');
$params->addOpt(':','U','user','<login>','Login or username');
$params->addOpt(':','P','password','<password/pathfile>','password or path for pubkey, sharedkey or special authentication options');
$params->addOpt(':','A','authfile','<filepath>','authentication file filepath form set default user, password parameters');
$params->addOpt(':','t','timeout','<second>','Timeout for script usage');

$params->addOpt(':','p','path','<path>','Path of remote script');
$params->addOpt(':','a','args','<arguments>','Aguments for remote script');

$params->addUsage('-H <host> -U <user> -P <password> -p <path> -a <arguments>');
$params->addUsage('-H <host> [-U <user>] -A <authfile> -p <path> -a <arguments>');

if( $params->getParam('help') )
{
    $params->print_help();
    exit($INT_STATUS['UNKNOWN']);
}

$o_path = $params->getParam('path');
$o_args = $params->getParam('args');

if( empty($o_path) )
{
    $params_opt_str = $params->getParamOptionsString('path');
    exit_state('UNKNOWN',"Missing required parameters $params_opt_str");
}

$is_debug_mode = $params->getParam('debug');

$o_authfile_path = $params->getParam('authfile');
if(!empty($o_authfile_path))
{
    $params_opt_str = $params->getParamOptionsString('authfile');
    if( !file_exists($o_authfile_path) )
    {
        exit_state('UNKNOWN',"Unable to found $params_opt_str=$o_authfile_path");
    }
    $auth_file_data = file_get_contents($o_authfile_path);
    if( empty($auth_file_data))
    {
        exit_state('UNKNOWN',"Unable to open $params_opt_str=$o_authfile_path");
    }
    $auth_file_data = json_decode($auth_file_data,TRUE);
    if( !is_array($auth_file_data))
    {
        exit_state('UNKNOWN',"Unable to parse $params_opt_str=$o_authfile_path");
    }
}

if( $is_debug_mode)
{
    echo "File $o_authfile_path opened\n";
}

$auth_params = array();
foreach(array('host','user','password') as $key)
{
    $auth_params[$key] = $params->getParam($key,@$auth_file_data[$key]);
    $params_opt_str = $params->getParamOptionsString($key);
    if( empty($auth_params[$key]))
    {
        exit_state('UNKNOWN',"Missing required option $params_opt_str or authfile parameter $key");
    }
}

//Armement timeout
declare(ticks = 1);
function sig_handler($signo)
{
        global $timeout;
        exit_state("UNKNOWN","Timeout after $timeout seconds");
}

$timeout = $params->getParam('timeout',$DEFAULT_TIMEOUT);

pcntl_alarm($timeout);
pcntl_signal(SIGALRM, "sig_handler");

if( $is_debug_mode)
{
    echo "Timeout settings is alarm:$timeout, conn: $conn_timeout, curl: $curl\n";
}

$o_url = $params->getParam('uri',$o_server_url);
$http_params['key'] = $o_key;
$http_params['action'] = 'run';
$http_params['proto'] = $DEFAULT_TYPE;
$http_params['hostaddr'] = $auth_params['host'];
$http_params['user'] = $auth_params['user'];
//print_r($http_params);
$http_params['authentication'] = $auth_params['password'];

try
{
    $command = "$o_path $o_args";
    $command = "\$hash = @{};";
    $command.= "\$hash.output=$($o_path $o_args);";
    $command.= "\$hash.ret=\$LASTEXITCODE;";
    $command.= "\$hash |  ConvertTo-Json";

    $client = new DaemonClientPwsh($o_url,$http_params,$timeout);
    $client->open();
    $from_host_data = $client->run($command);
    $client->close($from_host_data);
    $from_host_data = json_decode($from_host_data,TRUE);

    $output = trim($from_host_data['output'],"\n");
    echo "$output\n";
    if( $from_host_data['ret'] < 0 || $from_host_data['ret'] > 3 )
    {
        exit_state('UNKNOWN','Unknown retval : '.$from_host_data['ret']."\n");
    }
    exit($from_host_data['ret']);
}
catch(Exception $ex)
{
    exit_state('UNKNOWN',$ex->getMessage());
}

function exit_state($state,$msg=null,$usage=false){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];

        $ret=$state;
        if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
        else $state=$STR_STATUS[$ret];

        if( $msg != null )
        {
          $msg=trim($msg);
          echo "$state : $msg\n";
        }
        if( $usage ) print_usage(true);
        exit($ret);
}

class DaemonClientPwsh
{
    private $client;
    private $http_params;

    function __construct($o_url,$http_params,$timeout)
    {
        // Réglage des timeouts de http en fonction du timeout de l'alarm.
        $conn_timeout = 1;
        $curl_timeout = 1;
        if($timeout > 20)
        {
            $conn_timeout = intval($timeout*0.2);
            $curl_timeout = intval($timeout*0.75);
        }
        elseif( $timeout > 10 )
        {
            $conn_timeout = intval($timeout*0.3);
            $curl_timeout = $timeout - $conn_timeout - 1;
        }
        elseif( $timeout > 5)
        {
            $conn_timeout = intval($timeout*0.4);
            $curl_timeout = $timeout - $conn_timeout;
        }
        elseif( $timeout > 2)
        {
            $conn_timeout = intval($timeout*0.5);
            $curl_timeout = $timeout - $conn_timeout;
        }

        // Prépration de la connection http
        $client = new HttpClient($o_url);
        $client->setOptions(CURLOPT_CONNECTTIMEOUT,$conn_timeout);
        $client->setOptions(CURLOPT_TIMEOUT,$curl_timeout);
        $this->client = $client;
        $this->http_params = $http_params;
    }

    function open()
    {
        $this->client->open();
    }

    function run($command)
    {
        $http_params = $this->http_params;
        $http_params['command'] = $command;
        $client = $this->client;
        $raw_data = $client->post('',$http_params);
        $code = $client->getHttpCode();

        //Analyse des données reçues
        $data = json_decode($raw_data,TRUE);
        if( !is_array($data) )
        {
            throw new Exception("Code HTTP $code received. Empty or invalid data received from daemon.");
        }
        elseif(isset($data['code']) && $data['code'] != 200 )
        {
            throw new Exception(sprintf('Code %d - %s received from daemon. Reason : %s',$data['code'],$data['message'],$data['more_info']));
        }
        elseif( empty($data['stdout']) && !empty($data['stderr']))
        {
            throw new Exception(sprintf('Error %s received from host %s',$data['stderr'],$http_params['hostaddr']));
        }
        elseif( empty($data['stdout']))
        {
            throw new Exception("Unable to parse received json data from daemon : ".$raw_data);
        }
        return $data['stdout'];
    }


    function close()
    {
        $this->client->close();
    }
}

class Parameters
{
    private $data = array();
    private $options = NULL;
    private $params = NULL;
    private $usage = array();

    function __construct()
    {
    }

    function addUsage($row)
    {
        $this->usage[] = $row;
    }

    static function getArrayBiggestWordSize($arr){
    	$biggest_size = 0;
            foreach($arr as $word){
                    $cur_size = strlen($word);
                    if( $biggest_size < $cur_size){
                            $biggest_size = $cur_size;
                    }
            }
    	return $biggest_size;
    }

    function print_usage($alone=TRUE)
    {
        $script = basename(__FILE__);
        $left = 'usage : ';
        $size_left = strlen($left);

        foreach($this->usage as $row)
        {
            echo str_pad($left,$size_left) . "$script $row [OPTIONS]\n";
            $left = '';
        }
        echo"\n";
        if($alone)
        {
            $row = "--help for more help";
            echo str_pad($left,$size_left) . "$script $row\n";
        }
    }

    function print_help()
    {
        $this->print_usage(FALSE);
        $opt = array();
        $desc = array();
        foreach($this->data as $key => $row)
        {
            $str = "\t-" . $this->getParamOptionsString($key);
            if( !empty($row['value']) )
            {
                $str.= '=<'.trim($row['value'],'<>').">";
            }
            $opt[] = $str;
            $desc[] = $row['descr'];
        }
        $size_left = self::getArrayBiggestWordSize($opt);
        foreach($opt as $idx => $left)
        {
            echo str_pad($left,$size_left) . "  ".$desc[$idx]."\n";
        }
    }

    public function getParamOptionsString($long_opt)
    {
        $row = $this->data[$long_opt];
        return "-".$row['short'] . ", --" . $row['long'];
    }

    public function addOpt($optional,$short_opt,$long_opt,$value_desc,$descr,$default = NULL)
    {
        $row = array();
        $row['optional'] = $optional;
        $row['short'] = $short_opt;
        $row['long'] = $long_opt;
        $row['value'] = $value_desc;
        $row['descr'] = $descr;
        $row['default'] = $default;
        $this->data[$long_opt] = $row;
    }

    private function getopt()
    {
        if( $this->options != NULL )
        {
            return;
        }

        $short_opts = '';
        $long_opts = array();

        foreach($this->data as $row)
        {
            $short_opts.= $row['short'].$row['optional'];
            $long_opts[] = $row['long'].$row['optional'];
        }
        $this->options = getopt($short_opts, $long_opts);
        $this->params = array();
        foreach($this->data as $row)
        {
            $req_opt = $row['optional'];
            $long_opt = $row['long'];
            $short_opt =  $row['short'];
            $default_opt = $row['default'];
            $key_name = $row['long'];

            if( $req_opt == '' )
            {
                if( isset($this->options[$long_opt]) || isset($this->options[$short_opt] ) )
                {
                    $value = TRUE;
                }
                else
                {
                    $value = FALSE;
                }
            }
            elseif( isset($this->options[$long_opt]))
            {
                $value = $this->options[$long_opt];
            }
            elseif( isset($this->options[$short_opt] ))
            {
                $value = $this->options[$short_opt];
            }
            else
            {
                continue;
                $value = $default_opt;
            }
            $this->params[$key_name] = $value;
        }
    }

    public function getParam($name,$default=NULL)
    {
        $this->getopt();
        if(isset($this->params[$name]) && is_numeric($this->params[$name]) && $this->params[$name] == 0)
        {
            return 0;
        }
        elseif( $default != NULL && empty($this->params[$name]) )
        {
            return $default;
        }
        elseif( !isset($this->params[$name]))
        {
            return NULL;
        }
        elseif(empty($this->params[$name]))
        {
            return FALSE;
        }
        return $this->params[$name];
    }
}
?>
