#!/usr/bin/perl
#
use strict;
use warnings;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use vars qw($PROGNAME);
use Net::SNMP;
use Getopt::Long;
use ESIA::SNMP;
use ESIA::PLUGIN;
use Encode qw(decode encode);
#nom du programme & description
my $PROGNAME = "check_snmp_storage.pl";
my $PROGDESC = "Check storage space by SNMP";
my $PROGVERSION = "1.0";
#Chargement du timer par défaut et des erreurs
# a installer apt install libjson-perl
my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;

my $oid_descr = '1.3.6.1.2.1.25.2.3.1.3';
my $oid_size = '1.3.6.1.2.1.25.2.3.1.5';
my $oid_used = '1.3.6.1.2.1.25.2.3.1.6';
my $oid_alloc_units= '1.3.6.1.2.1.25.2.3.1.4';



ESIA::PLUGIN::arm_end_signal($TIMEOUT);
Getopt::Long::Configure( "bundling" );
my ($opt_h,$opt_V);
my $warning=80;
my $critical=90;
sub ThresholdIsByte
{
	my $thres = 0;
	my $test =$_[0];
	if($test=~/([0-9]*)([K|M|G|T]B)$/)
	{
		my $value=$1;
		my $unit=$2;
		if($unit eq 'GB')
		{
			$thres=$value*1073741824;
		}
		elsif($unit eq 'MB')
		{
			$thres=$value*1024*1024;
		}
		elsif($unit eq 'TB')
		{
			$thres=$value*1099511627776;
		}
		else
		{
			$thres=$value*1024;
		}
	}
	return $thres;
}
my $match=".*";
my $reserved=0;
my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
		"V"   => \$opt_V,	 "version"    => \$opt_V,
		"h"   => \$opt_h,	 "help"       => \$opt_h,
		"m=s" => \$match,	 "match=s" => \$match,
		"w=s" => \$warning,	 "warning=s" => \$warning,
		"c=s" => \$critical, "critical=s" => \$critical,
        "R=i" => \$reserved, "reserved=i" => \$reserved,
	  );


# -h & --help print help
if ($opt_h || !defined($snmp))
{
    #affiche l'aide par défaut
	ESIA::SNMP::print_snmp_usage($PROGNAME,'-m namestorage');
	my @arr;
	push(@arr,["-m","--match","name of storage (regex compatible)" ]);
	push(@arr,["-w","--warning","warning threshold in percent for used space (default: 80%) specified MB, GB, TB to test remaining space rather than percentage (ex: -w 5GB)" ]);
	push(@arr,["-c","--critical","critical threshold in percent for used space (default: 90%) specified MB, GB, TB to test remaining space rather than percentage (ex: -c 2GB)"]);
	push(@arr,["-R","--reserved","If you are using ext4 partitions, you can specify the reservation value of your partition. 5% by default. To activate this function, use the argument --reserved=5"]);
	ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
	exit $ERRORS{'OK'};
}
# -V & --version print version
if ($opt_V)
{
	print "$PROGNAME version $PROGVERSION\n";
	exit $ERRORS{'OK'};
}
$snmp->translate([-octetstring => 0]);
$warning =~ s/\%//g;
$critical =~ s/\%//g;
my $Sysoid_value = ESIA::SNMP::getManufacturerOID($snmp);
if($Sysoid_value==-1)
{
	print "UNKNOWN: Host not SNMP response $snmperror\n";
	exit $ERRORS{'UNKNOWN'};
}
my $return = 0;
my $flag_match=0;
my $result = $snmp->get_table( $oid_descr);
my $graphdata="";
my $warning_byte=ThresholdIsByte($warning);
my $critical_byte=ThresholdIsByte($critical);

foreach my $key ( keys %$result)
{
    my $name = $$result{$key};
    utf8::encode($name);
    if($name =~/$match/)
    {
        $flag_match=1;
		if($key=~/\.([0-9]*)$/)
		{
            my $id = $1;
            my $used = ESIA::SNMP::get($snmp,"$oid_used.$id");
            my $size = ESIA::SNMP::get($snmp,"$oid_size.$id");
            my $unit = ESIA::SNMP::get($snmp,"$oid_alloc_units.$id");
            my $percent_used = 0;
            my $used_byte = $used * $unit;
            my $size_byte = $size * $unit;
            
            my $reserved_message="";
            if($reserved != 0)
            {
                my $rese=($size_byte/100)*$reserved;
                $size_byte=$size_byte-$rese;

                my $rese_B =sprintf("%.0f",$rese/1024/1024);
                my $unitB ='MB';
                if(int($rese/1099511627776) > 0 )
                {
				    $rese_B =sprintf("%.1f",$rese/1099511627776);
				    $unitB ='TB';
                }
                elsif(int($rese/1073741824) > 0 )
			    {
				    $rese_B =sprintf("%.1f",$rese/1073741824);
                	$unitB ='GB';
                }
                $reserved_message="Reserved: $rese_B$unitB";
             #   $used_byte=$used_byte+$rese;
            }

            my $used_MB =sprintf("%.0f",$used_byte/1024/1024);
            my $size_MB =sprintf("%.0f",$size_byte/1024/1024);
            my $unitB ='MB';
            if(int($used_byte/1099511627776) > 0 )
            {
				$used_MB =sprintf("%.1f",$used_byte/1099511627776);
				$size_MB =sprintf("%.1f",$size_byte/1099511627776);
				$unitB ='TB';
            }
            elsif(int($used_byte/1073741824) > 0 )
			{
				$used_MB =sprintf("%.1f",$used_byte/1073741824);
            	$size_MB =sprintf("%.1f",$size_byte/1073741824);
            	$unitB ='GB';
            }
            if($size != 0)
            {
                $percent_used = ($used_byte/$size_byte)*100;
            }
            $percent_used = sprintf("%.0f",$percent_used);
		    my $message = "";
		    my $state = "OK";
			if($warning_byte == 0)
			{
				$message = "< $warning%";
		    	if($percent_used >= $warning)
				{
					$state = "WARNING";
					$message = "> $warning%";
		    	}
		    }
			else
			{
				my $free= $size_byte - $used_byte;
				if($free < $warning_byte)
				{
					$state = "WARNING";
					$message = "free space less than $warning";
				}
			}
			if($critical_byte == 0)
			{
		    	if($percent_used >= $critical)
				{
					$state = "CRITICAL";
					$message = "> $critical%";
		    	}
		    }
			else
			{
				my $free= $size_byte - $used_byte;
				if($free < $critical_byte)
				{
					$state = "CRITICAL";
					$message = "free space less than $critical";
				}
			}

            print "$state: $name: ".$percent_used."%"."used($used_MB"."$unitB/$size_MB"."$unitB) $message. $reserved_message\n";
            $graphdata="used=$percent_used;usedByte=$used_byte;totalByte=$size_byte";
            $return = ESIA::PLUGIN::priority_state($return,$ERRORS{$state});
        }
    }
}
if($flag_match==0)
{
    print"UNKNOWN: partition $match not found\n";
    $return = $ERRORS{'UNKNOWN'};
}
else
{
    print"|$graphdata";
}
exit $return;
