#!/usr/bin/perl

use strict;
use lib "/usr/local/esia/plugins/lib/perl";
use DateTime;
use DateTime::Format::Strptime;
use Getopt::Long;
use Net::SNMP;
use ESIA::SNMP;
use ESIA::PLUGIN;

my $TIMEOUT = $ESIA::PLUGIN::TIMEOUT;
my %ERRORS= %ESIA::PLUGIN::ERRORS;
my $critical=60;
my $help=undef;
my $timezone=undef;
my $warning=30;
my $debug;
my $local_time="";
my $local_tz="";
my $parameters="";
my $exit_code=0;

my $PROGNAME='check_snmp_time.pl';
my $PROGDESC='check TIME.';

my $return=0;
my $output = "";
ESIA::PLUGIN::arm_end_signal($TIMEOUT);
sub check_options
{
	if (defined ($help))
	{
        #affiche l'aide par défaut
        print "\nNagios plugin to check the time on a server using SNMP.\n";
        print "GPL licence, (c)2007 Karl Bolingbroke\n";
	    print "Version 1.02\n\n";
	    print "This SNMP OID is known to work on various versions of Linux and Windoze hosts,\n";
	    print "and is known to NOT work on HP-UX 11.11.\n";
        ESIA::SNMP::print_snmp_usage($PROGNAME,'-Z timzone');
        my @arr;
        push(@arr,["-w","--warning","warning threshold temperature in Celsius (default: 80) or warning threshold temperature in Celsius (default: 50)" ]);
        push(@arr,["-c","--critical","Number of seconds the time can be off before giving critical alert. (Default: 60)"]);
        push(@arr,["-S","--snmpver","Version of SNMP protocol to use: 1|2c|3. (Default: 2c)"]);
        push(@arr,["-l","--login","Login for snmpv3 authentication (implies v3 protocol with MD5)"]);
        push(@arr,["-Z","--timezone","Timezone of the remote host.  This is necessary, because Windows hosts don't report the timezone."]);
        ESIA::SNMP::print_snmp_help($PROGNAME,$PROGDESC,\@arr);
        exit $ERRORS{'OK'};
    }
	if ( !defined($timezone) )
	{
		print "Missing timezone of the remote host!\nThis is required because Windoze hosts don't report their timezone.\n";
        ESIA::SNMP::print_snmp_usage($PROGNAME,'-Z timezone');
		exit $ERRORS{"UNKNOWN"}
	}
}
Getopt::Long::Configure( "bundling" );

my ($snmp, $snmperror) = ESIA::SNMP::SessionAutoCommunityVersion(@ARGV);
GetOptions(
		'D'     => \$debug,             'debug'         => \$debug,
        'h'     => \$help,              'help'          => \$help,
        'w=i'   => \$warning,           'warning=i'  	=> \$warning,
        'c=i'   => \$critical,          'critical=i'    => \$critical,
        'Z=s'   => \$timezone,          'timezone=s'    => \$timezone
   );
check_options();
$snmp->translate([-octetstring => 0]);
my $remote_host= $snmp->hostname();
my $remote_ts = ESIA::SNMP::get($snmp,"1.3.6.1.2.1.25.1.2.0");
if (!defined($remote_ts))
{
    printf("UNKNOWN ERROR: Description table : %s.\n", $snmp->error);
    $snmp->close;
    exit $ERRORS{"UNKNOWN"};
}

my @date = unpack 'n C6 a C2', $remote_ts;
my $formatter = DateTime::Format::Strptime->new(pattern=>"%s");
my $local_ts=DateTime->now(time_zone=>"local",formatter =>$formatter);
if (defined $debug) { print "$local_ts|$remote_ts"; }
$remote_ts=DateTime->new( year=>$date[0], month=>$date[1], day=>$date[2], hour=>$date[3], minute=>$date[4], second=>$date[5], time_zone=>$timezone,formatter =>$formatter);

if (defined $debug) { print "$local_ts|$remote_ts"; }

my $diff=int($local_ts)-int($remote_ts);
if (abs($diff) > $critical)
{
    $output .= "CRITICAL: $remote_host time is off by $diff seconds!\n";
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{CRITICAL});
}
elsif ($diff > $warning)
{
    $output .= "WARNING: $remote_host time is off by $diff seconds!\n";
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{WARNING});
}
else
{
    $output .= "OK: $remote_host time differs by $diff seconds.\n";
	$return = ESIA::PLUGIN::priority_state($return,$ERRORS{OK});
}

if($output eq "")
{
    print "UNKNOWN: Status retrieval failed";
    exit $ERRORS{UNKNOWN};
}
else
{
    print $output;
    exit $return;
}
