package ESIA::CacheFile;
use warnings;
use strict;
use utf8;
use Carp;
use Unicode::Normalize;
use Fcntl ':flock';
use JSON;
#constructeur
sub new
{

    my ($classe,$path, $filename,$during ) = @_;
    $classe = ref($classe) || $classe;
    if($path=~/\/$/)
	{
        #print OK: on a le / final
    }
    else
    {
        $path.='/';
    }
    #test si le répertoire existe
    if (! -d $path)
    {
        mkdir($path);
    }
    if(!defined $during)
    {
        $during=30;
    }
    my $this = {};
    bless( $this, $classe );
    $filename = NFD($filename);
    $filename =~ s/[^[:ascii:]]//g;
    $filename =~ s/[^A-Za-z0-9]/_/g;
    my $fullpath =   $path.$filename;
    $this->{_PATH}  = $fullpath;
    $this->{_FLAG}  = "";
    $this->{_TIME}  = -1;
    $this->{_DURING}= $during;
    $this->{_DATA}  = "";
    #test si le fichier existe
    if ( -e $fullpath)
    {
        #le fichier existe
        open(my $fh, "<",  $this->{_PATH}) or die "CacheFile '$fullpath' can't be opened";
        if (flock($fh, LOCK_SH))
        {
            my $json ="";
            while(<$fh>)
            {
               $json.= $_;
            }
            flock($fh, LOCK_UN);    # Enlève le verrou
            my $content = eval { decode_json($json) };
            if($@)
            {
                 $this->{_FLAG}  = 'Bad\'t json file';
            }
            else
            {
                $this->{_DATA} =  $content->{data};
                $this->{_TIME} = $content->{time};
                $this->{_DURING} = $content->{during};
            }
        }
        else
        {
            $this->{_FLAG}  = 'Can\'t lock file';
        }
        close($fh);
    }

    return $this;
}
# Méthode parler - un argument
sub isValid
{
    my $this = shift;
    if($this->{_FLAG} ne "")
    {
        return 0;
    }
    my $now = time();
    my $outimestamp = $this->{_TIME} + $this->{_DURING}*60;
    if($this->{_TIME} == -1)
    {
        return 0;
    }
    elsif($outimestamp <= $now)
    {
        return 0;
    }
    else
    {
        return 1;
    }
}
sub getData
{
    my $this = shift;
    return $this->{_DATA};
}
sub setData
{
    my ( $this, $data,$time ) = @_;
    $this->{_DATA} = $data;
    if(defined $time)
    {
        $this->{_TIME} = $time;
    }
    else
    {
        $this->{_TIME} =time();
    }
    return;
}
sub writeData()
{
    my ( $this, $data,$time ) = @_;
    $this->{_DATA} = $data;
    if(defined $time)
    {
        $this->{_TIME} = $time;
        $this->{_DURING}=0;
    }
    else
    {
        $this->{_TIME} =time();
    }
    my %record = ('time'=> $this->{_TIME},'during'=>$this->{_DURING},'data'=>$this->{_DATA});
    my $json = encode_json \%record;
    #le fichier existe
    open(my $file, "+>",  $this->{_PATH}) or die "CacheFile ".$this->{_PATH}." can't be opened";
    if (flock($file, LOCK_EX))
    {
         #   print "JSON: $json";
        print $file "$json\n";
        flock($file, LOCK_UN);    # Enlève le verrou
    }
    else
    {
         print 'Can\'t lock file';
        $this->{_FLAG}  = 'Can\'t lock file';
    }
    close($file);
    return 1;
}
1;
__END__
