# --- fichier INVENTORY.pm ---
package ESIA::INVENTORY;
use lib "/usr/local/esia/plugins/lib/perl"  ;
use strict;
use warnings;
use Net::SNMP;
use ESIA::SNMP;
use JSON;
our $JSON = 1;
our $PRETTY = 2;
our $ARRAY = 3;
our %ENTITY_PHYSICAL_CLASS  = ( "1"  => "other",
                               "2"  => "unknown",
                               "3"  => "CHASSIS",
                               "4"  => "backplane",
                               "5"  => "container",
                               "6"  => "powerSupply",
                               "7"  => "fan",
                               "8"  => "sensor",
                               "9"  => "module",
                               "10"  => "port",
                               "11"  => "stack",
                               "12"  => "cpu",
                        );
our %IF_TYPE_CLASS  = ( "1"  => "other",
                               "2"  => "regular1822",
                               "3"  => "hdh1822",
                               "4"  => "ddn-x25",
                               "5"  => "rfc877-x25",
                               "6"  => "ethernet",
                               "7"  => "iso88023",
                               "8"  => "tokenBus",
                               "9"  => "tokenRing",
                               "10"  => "man",
                               "11"  => "starLan",
                               "12"  => "proteon-10Mbit",
                               "13"  => "proteon-80Mbit",
                               "14"  => "hyperchannel",
                               "15"  => "fddi",
                               "16"  => "lapb",
                               "17"  => "sdlc",
                               "18"  => "ds1",
                               "19"  => "e1",
                               "20"  => "ISDN",
                               "21"  => "ISDN",
                               "22"  => "propPointToPointSerial",
                               "23"  => "ppp",
                               "24"  => "softwareLoopback",
                               "25"  => "eon",
                               "26"  => "ethernet-3Mbit",
                               "27"  => "nsip",
                               "28"  => "slip",
                               "29"  => "ultra",
                               "30"  => "ds3",
                               "31"  => "sip",
                               "32"  => "frame-relay"
                        );
sub getDefaultChassisInventoryMib
{
    my($snmp,$type,$excludetype) = @_;
    my @output;
    #CHASSIS
    my $oid_phy_name = "1.3.6.1.2.1.47.1.1.1.1.2";
    my $oid_phy_type = "1.3.6.1.2.1.47.1.1.1.1.5";
    my $oid_phy_serial= "1.3.6.1.2.1.47.1.1.1.1.11";
    my $oid_phy_manufacturer = "1.3.6.1.2.1.47.1.1.1.1.12";
    my $oid_phy_model= "1.3.6.1.2.1.47.1.1.1.1.13";
    my $oid_phy_hardware_version= "1.3.6.1.2.1.47.1.1.1.1.8";
    my $oid_phy_firmware_version= "1.3.6.1.2.1.47.1.1.1.1.9";
    my $result = $snmp->get_table( $oid_phy_model);
    foreach my $key ( sort keys %$result)
    {
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $index =$1;
            my $model =$$result{"$key"};
            if($model ne "" && $model ne 'N/A')
            {
                my $serial = ESIA::SNMP::get($snmp,"$oid_phy_serial.$index");
                my $type = ESIA::SNMP::get($snmp,"$oid_phy_type.$index");
                $type = $ENTITY_PHYSICAL_CLASS{$type};
                my $name = ESIA::SNMP::get($snmp,"$oid_phy_name.$index");
                my $hardwareversion = ESIA::SNMP::get($snmp,"$oid_phy_hardware_version.$index");
                my $firmware = ESIA::SNMP::get($snmp,"$oid_phy_firmware_version.$index");
                my $manufacturer = ESIA::SNMP::get($snmp,"$oid_phy_manufacturer.$index");
			    my %hash = ('index'=>$index, 'name' => $name,'manufacturer'=>$manufacturer, 'model' => $model, 'type'=>$type,'serial'=>$serial,'hardwareVersion' => $hardwareversion, 'firmwareVersion' => $firmware);

			    push(@output, \%hash);
            }
        }
    }
    if($type == $ESIA::INVENTORY::JSON)
    {
        my $out = encode_json(\@output);
        return $out;
    }
    elsif($type == $ESIA::INVENTORY::ARRAY)
    {
        return \@output;
    }
    else
    {
        my $size = @output;
        if($size != 0)
        {
            foreach my $line ( @output)
            {

                my $hash = \%$line;
                print "name: ".$hash->{name}.", manufacturer: ".$hash->{manufacturer}.", model: ".$hash->{model}.", type: ".$hash->{type}.", serial: ".$hash->{serial};
                print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
            }
            return 0;
        }
        else
        {
            return 1;
        }
    }
}
sub getDefaultHardwareMib
{
    my($snmp,$type) = @_;
    my @output;
    #CPU -- HOST RESSOURCE MIB
    my $oid_host_hard_type = "1.3.6.1.2.1.25.3.2.1.2";
    my $oid_host_hard_descr = "1.3.6.1.2.1.25.3.2.1.3";
    my $oid_host_disk_capacity = "1.3.6.1.2.1.25.3.6.1.4";
    my $result = $snmp->get_table( $oid_host_hard_type);
    foreach my $key ( sort keys %$result)
    {
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my %option;
            my $index = $1;
            my $hardtype = $$result{"$key"};
            if($hardtype =~ /3.6.1.2.1.25.3.1.([0-9]*)$/)
            {
                $hardtype=$1;
            }
            if( $hardtype != 4 && $hardtype != 12)
            {
                my $descr = ESIA::SNMP::get($snmp,"$oid_host_hard_descr.$index");
                my $hardtype_descr ='unknown';
                my $manufacturer = '';

                if($hardtype ==3)
                {
                    #Processor
                    $hardtype_descr='CPU';
                    if($descr =~/GenuineIntel\:\s+(.*)/)
                    {
                        $manufacturer = 'Intel';
                        $descr=$1;
                    }
                    elsif($descr =~/Intel/)
                    {
                        $manufacturer = 'Intel';
                    }
                    elsif($descr =~/AMD/)
                    {
                        $manufacturer = 'AMD';
                    }
                }
                elsif($hardtype ==5)
                {
                    #Printer
                    $hardtype_descr='PRINTER';
                }
                elsif($hardtype ==6)
                {
                    #HDD
                    $hardtype_descr='HDD';
                    my $capacity = ESIA::SNMP::get($snmp,"$oid_host_disk_capacity.$index");
                    $option{capacity}=$capacity;
                }
                elsif($hardtype ==13)
                {
                    #Printer
                    $hardtype_descr='KEYBOARD';
                }
                my %hash = ('name' => $descr,'manufacturer'=>$manufacturer, 'model' => $descr, 'type'=>$hardtype_descr,'hardwareVersion' => '', 'firmwareVersion' => '','option'=>\%option);
                push(@output, \%hash);
            }
        }
    }
    #NETWORK -- IF MIB
    my $oid_ifName=".1.3.6.1.2.1.2.2.1.2";
    my $oid_ifPhysAddress=".1.3.6.1.2.1.2.2.1.6";
	my $oid_ifType=".1.3.6.1.2.1.2.2.1.3";
    my $oid_IPAddress=".1.3.6.1.2.1.4.20.1.1";
    my $oid_ifIndex=".1.3.6.1.2.1.4.20.1.1";
    my $resultIP = $snmp->get_table( $oid_ifIndex);
    $result = $snmp->get_table( $oid_ifName);
    my $manufacturer='';
    foreach my $key ( sort keys %$result)
    {
        #on récupère l'index
        if($key=~ /([0-9]*)$/ )
        {
            my $index = $1;
            my %option;
            my $ifname = $$result{"$key"};
            my $ifType = ESIA::SNMP::get($snmp,"$oid_ifType.$index");
            my $mac = ESIA::SNMP::get($snmp,"$oid_ifPhysAddress.$index");
            $option{macAdress}=$mac;
            my $ip="";
            my $tmp = $IF_TYPE_CLASS{$ifType};
            if($tmp)
            {
            	$ifType = $tmp;
            }
            else
            {
            	$ifType='network';
            }
            if($resultIP)
            {
                foreach my $keyip ( sort keys %$resultIP)
                {
                    my $id = $$result{"$keyip"};
                    if($id)
                    {
                        if($index == $id)
                        {
                            if($key=~ /3.6.1.2.1.4.20.1.2\.([0-9\.]*)$/ )
                            {
                                $ip = ESIA::SNMP::get($snmp,"$oid_IPAddress.$1");
                            }
                        }
                    }
                }
            }
            $option{IP}=$ip;

            my %hash = ('name'=>$ifType,'fullname' => $ifname,'manufacturer'=>$manufacturer, 'model' => '', 'type'=>'NETINT','hardwareVersion' => '', 'firmwareVersion' => '','option'=>\%option);
            push(@output, \%hash);
        }
    }

    if($type == $ESIA::INVENTORY::JSON)
    {
        my $out = encode_json(\@output);
        return $out;
    }
    elsif($type == $ESIA::INVENTORY::ARRAY)
    {
        return \@output;
    }
    else
    {
        my $size = @output;
        if($size != 0)
        {
            foreach my $line ( @output)
            {

                my $hash = \%$line;
                print "name: ".$hash->{name}.", model: ".$hash->{model}.", type: ".$hash->{type};
                print ", hardwareVersion: ".$hash->{hardwareVersion}.", firmwareVersion: ".$hash->{firmwareVersion}."\n";
            }
            return 0;
        }
        else
        {
            return 1;
        }
    }
}
