# --- fichier PLUGIN.pm ---
package ESIA::PLUGIN;
use strict;
use warnings;
use POSIX qw(floor);
our $TIMEOUT = 50;
our %ERRORS=('OK'=>0,'WARNING'=>1,'CRITICAL'=>2,'UNKNOWN'=>3,'DEPENDENT'=>4);
our @priorities=(
                $ERRORS{CRITICAL},
                $ERRORS{WARNING},
                $ERRORS{UNKNOWN},
                $ERRORS{OK});
sub priority_state
{
    my($a,$b) = @_;
    if( $a == $b )
    {
         #pas besoin d'aller dans la boucles si ils sont identiques
        return $a;
    }
    foreach my $v (@ESIA::PLUGIN::priorities)
    {
        if ( $v == $a || $v == $b )
        {
            return $v;
        }
    }
}
sub arm_end_signal
{
    my($a) = @_;
    $SIG{'ALRM'} = sub {
	    print ("ERROR: Plugin took too long to complete (alarm)\n");
	    exit $ERRORS{"UNKNOWN"};
    };
    alarm($a);
}
sub valueByTime
{
    my($time1,$value1,$time2,$value2) = @_;
	my $delta = $time1-$time2;
	if($delta == 0)
	{
		return 0;
	}
	my $delta_value = $value1 - $value2;
	$delta_value = $delta_value/$delta;
	return $delta_value;
}
sub IntegerValueByTime
{
    my($time1,$value1,$time2,$value2) = @_;
	my $delta = $time1-$time2;
	if($delta == 0)
	{
		return 0;
	}
	my $delta_value = $value1 - $value2;
	$delta_value = $delta_value/$delta;
	return int($delta_value);
}
sub valueIsInError
{
	my $bandwith =  $_[0];
    my $threshold = $_[1];
	my $threshold_low =  -1000;
	my $threshold_high =  -1;
    my $ret = 0;
    my $message = "";
    #test des valeurs critique
    if($threshold =~/([0-9\.]*)\:([0-9\.]*)/)
    {
        $threshold_low=$1;
        $threshold_high=$2;
    }
    elsif($threshold =~/([0-9\.]*)\,([0-9\.]*)/)
    {
        $threshold_low=$1;
        $threshold_high=$2;
    }
    else
    {
        $threshold_high=$threshold;
    }
    #print "THRESHOLD  $threshold_low <> $threshold_high\n";
    my $trigger="";
    my $op="";
    if($bandwith > $threshold_low && $bandwith < $threshold_high)
    {
    	$ret = 0;
    }
    elsif($bandwith  > $threshold_high)
    {
    	$op=">";
    	$ret = 1;
    	$trigger=$threshold_high;
    }
    elsif($bandwith < $threshold_low && $threshold_low != -1000 )
    {
    	$trigger=$threshold_low;
    	$op="<";
    	$ret = -1;
    }
	return ($ret, $op,$trigger);
}
sub print_header
{
    my $progname = $_[0];
    my $progdescr= $_[1];
	print "Copyright (c) 2019 Esia SA\n";
	print "\n";
	print "$progname\n";
    print '=' x length($progname);
	print "\n";
    if($progdescr)
    {
        print "Description:\n";
	    print "     $progdescr\n";
        print '=' x length($progname);
	    print "\n";
    }
}
sub print_help
{
    my $used = $_[0];
    my $othervar =  $_[1];
    if($othervar)
    {
        foreach my $key ( @$othervar)
        {
            if($key eq 'V')
            {
                push(@$used,["-V","--version","plugin version"]);
            }
            elsif($key eq 'h')
            {
                push(@$used,["-h","--help","print this help"]);
            }
        }
    }
    print "Plugin parameters: \n";
    printf ("     %-10s%-20s%s\n", 'short','long','description:');
    foreach my $key ( @$used)
    {
        my @line = @$key;
        printf ("     %-10s%-20s%s\n", $line[0],$line[1],$line[2]);
    }
}
1;
